<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

final class WFFN_Header {

	public $data = [];

	/**
	 * Constructor.
	 * @since  1.0.0
	 */
	public function __construct() {
		$this->data['back_link']                            = '';
		$this->data['back_link_label']                      = __( 'Funnels', 'funnel-builder' );
		$this->data['level_1_navigation_active']            = '';
		$this->data['level_2_title']                        = '';
		$this->data['level_2_post_title']                   = '';
		$this->data['level_2_right_wrap_type']              = 'menu';
		$this->data['level_2_right_side_navigation']        = [];
		$this->data['level_2_navigation_pos']               = 'left';
		$this->data['level_2_right_side_navigation_active'] = '';
		$this->data['level_2_right_html']                   = '';
	}

	public function set_back_link( $enable = 0, $link = '' ) {
		if ( 0 === $enable || empty( $link ) ) {
			return;
		}
		$this->data['back_link'] = $link;
	}

	public function set_level_1_navigation_active( $active = '' ) {
		if ( empty( $active ) ) {
			return;
		}
		$this->data['level_1_navigation_active'] = $active;
	}

	public static function left_navigation() {
		return [
			'dashboard' => [
				'name' => 'Dashboard',
				'link' => admin_url( 'admin.php?page=bwf' ),
			],
			'funnels'  => [
				'name' => 'Funnels',
				'link' => admin_url( 'admin.php?page=bwf&path=/funnels' ),
			],
			'settings' => [
				'name' => 'Settings',
				'link' => admin_url( 'admin.php?page=bwf&path=/settings' ),
			],
		];
	}

	private static function prepare_web_url( $link ) {
		return add_query_arg( [
			'utm_source'   => 'website',
			'utm_medium'   => 'text',
			'utm_campaign' => 'header',
		], $link );
	}

	public static function right_navigation() {

		return [
			'help'      => [
				'name'   => __( 'Read Docs', 'funnel-builder' ),
				'desc'   => __( 'Get help along the way', 'funnel-builder' ),
				'icon'   => 'help',
				'link'   => self::prepare_web_url( 'https://buildwoofunnels.com/documentation/' ),
				'target' => '_blank'
			],
			'support'   => [
				'name'   => __( 'Get Help', 'funnel-builder' ),
				'desc'   => __( 'Contact support team', 'funnel-builder' ),
				'icon'   => 'support',
				'link'   => self::prepare_web_url( 'https://buildwoofunnels.com/support' ),
				'target' => '_blank'
			]
		];
	}

	/**
	 * For React Menu Render
	 */
	public function get_render_data() {
		return [
			'logo'           => esc_url( plugin_dir_url( WFFN_PLUGIN_FILE ) . 'admin/assets/img/menu/WooFunnels-Logo.svg' ),
			'logo_link'      => admin_url( 'admin.php?page=bwf' ),
			'left_nav'       => self::left_navigation(),
			'right_nav'      => self::right_navigation(),
			'data'           => $this->data,
			'pluginDir'      => WFFN_PLUGIN_FILE
		];
	}


	public function render() {
		ob_start();
		?>
        <style>
            div#wpcontent {
                padding: 0;
                overflow-x: hidden !important;
                min-height: calc(100vh - 32px);
            }
        </style>
        <div class="bwfan_header">
            <div class="bwfan_header_l1">
                <div class="bwfan_header_l">
                    <div>
                        <a class="bwf-breadcrumb-svg-icon" href="<?php echo esc_url( admin_url( 'admin.php?page=bwf' )); ?>">
                            <img src="<?php echo esc_url( plugin_dir_url( WFFN_PLUGIN_FILE ) . 'admin/assets/img/menu/WooFunnels-Logo.svg' ) ?>" alt="WooFunnels Funnel Builder"/>
                        </a>
                    </div>
                    <div class="bwfan_navigation">
						<?php
						$navigation = self::left_navigation();
						$this->output_navigation( $navigation, $this->data['level_1_navigation_active'] );
						?>
                    </div>
                </div>
                <div class="bwfan_header_r">
                    <div class="bwfan_navigation bwf-header-ellipses-wrap">
						<?php
						$navigation = self::right_navigation();
						$this->outputEllipsisMenu( $navigation );
						?>
                    </div>
                </div>
            </div>
			<div class="bwfan_page_header"><?php esc_html_e( 'Funnels', 'funnel-builder' ) ?></div>
			<div class="bwfan_header_l2">
                <div class="bwfan_header_l">
					<?php

					if ( ! empty( $this->data['back_link'] ) ) {
						echo '<span class="bwfan_header_l2_back"><a href="' . esc_url( $this->data['back_link']) . '">' . $this->data['back_link_label'] . '</a></span>';//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					}
					echo '<span class="bwfan_header_l2_wrap">';
					if ( ! empty( $this->data['level_2_title'] ) ) {
						echo '<span id="bwfan_automation_name">' . $this->data['level_2_title'] . '</span>';//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					}
					if ( ! empty( $this->data['level_2_post_title'] ) ) {
						echo $this->data['level_2_post_title'];//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					}
					echo '</span>';
					if ( ( 'menu' === $this->data['level_2_right_wrap_type'] || 'both' === $this->data['level_2_right_wrap_type'] ) && 'right' !== $this->data['level_2_navigation_pos'] ) {
						$navigation = $this->data['level_2_right_side_navigation'];
						$active     = $this->data['level_2_right_side_navigation_active'];

						echo '<div class="bwfan_navigation">';
						$this->output_navigation( $navigation, $active );
						echo '</div>';
					}
					?>
                </div>
                <div class="bwfan_header_r">
					<?php
					if ( 'right' === $this->data['level_2_navigation_pos'] ) {
						$navigation = $this->data['level_2_right_side_navigation'];
						$active     = $this->data['level_2_right_side_navigation_active'];

						echo '<div class="bwfan_navigation">';
						$this->output_navigation( $navigation, $active );
						echo '</div>';
					}
					if ( ( 'html' === $this->data['level_2_right_wrap_type'] || 'both' === $this->data['level_2_right_wrap_type'] ) && ! empty( $this->data['level_2_right_html'] ) ) {
						echo $this->data['level_2_right_html'];//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
					}
					?>
                </div>
            </div>
			<hr class="wp-header-end" />
        </div>
		<?php
		return ob_get_clean();
	}

	public function output_navigation( $navigation, $active_slug = '' ) {

		if( 'string' === gettype( $navigation ) ) {
			echo $navigation;//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			return;
		}

		if ( ! is_array( $navigation ) || 0 === count( $navigation ) ) {
			return;
		}
		foreach ( $navigation as $key => $item ) {
			$active = ( ! empty( $active_slug ) && $key === $active_slug ) ? 'bwfan_navigation_active' : '';

			$menu_title = isset( $item['name'] ) ? $item['name'] : $item['title'];

			$icon = '';

			$target = ( isset( $item['target'] ) && ! empty( $item['target'] ) ) ? ' target="' . $item['target'] . '"' : '';

			echo '<span><a href="' . esc_url($item['link']) . '" class="' . esc_attr($active) . '"' . $target . '>' .$icon . $menu_title .'</a></span>';//phpcs:ignore
		}
		$active = null;
	}

	public function outputEllipsisMenu( $navigation ) {
		if ( ! is_array( $navigation ) || 0 === count( $navigation ) ) {
			return;
		}
		?>
        <div class="bwf-ellipsis-menu bwf-ellipsis--alter-alter">
            <div class="components-dropdown">
                <button type="button" title="Quick Actions" aria-expanded="false" class="components-button bwf-ellipsis-menu__toggle has-text has-icon">
                    <svg width="25" height="25" viewBox="4 4 20 20" version="1.1" xmlns="http://www.w3.org/2000/svg">
                        <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">
                            <g id="4.-Automation-Task-Splitup-and-Help-Icon" transform="translate(-1369.000000, -15.000000)">
                                <g id="Group-6" transform="translate(1370.000000, 16.000000)">
                                    <circle id="Oval" stroke="#FFFFFF" cx="13.5" cy="13.5" r="13.5"></circle>
                                    <g id="icons8-help" transform="translate(5.000000, 5.000000)" fill="#202B31" fill-rule="nonzero" opacity="0.74">
                                        <path d="M8.5,0 C3.81312805,0 0,3.81313145 0,8.5 C0,13.1868685 3.81312805,17 8.5,17 C13.1868719,17 17,13.1868685 17,8.5 C17,3.81313145 13.1868719,0 8.5,0 Z M8.5,1.275 C12.4978114,1.275 15.725,4.50219117 15.725,8.5 C15.725,12.4978088 12.4978114,15.725 8.5,15.725 C4.50218862,15.725 1.275,12.4978088 1.275,8.5 C1.275,4.50219117 4.50218862,1.275 8.5,1.275 Z M8.5,3.825 C7.09925737,3.825 5.95,4.97425737 5.95,6.375 L5.95,6.5875 C5.94674857,6.8174054 6.06753925,7.03125187 6.26611894,7.14715342 C6.46469863,7.26305497 6.71030137,7.26305497 6.90888106,7.14715342 C7.10746075,7.03125187 7.22825143,6.8174054 7.225,6.5875 L7.225,6.375 C7.225,5.66349262 7.78849262,5.1 8.5,5.1 C9.21150737,5.1 9.775,5.66349262 9.775,6.375 C9.775,7.36988632 9.516328,7.49455157 9.06279307,7.8508788 C8.8360254,8.02904262 8.5389997,8.23446085 8.28500992,8.58051752 C8.03101972,8.9265742 7.8625,9.40896535 7.8625,9.9875 C7.85924857,10.2174054 7.98003925,10.4312519 8.17861894,10.5471534 C8.37719863,10.663055 8.62280137,10.663055 8.82138106,10.5471534 C9.01996075,10.4312519 9.14075143,10.2174054 9.1375,9.9875 C9.1375,9.62487215 9.20804277,9.47757905 9.31264632,9.3350587 C9.4172503,9.19253792 9.5983496,9.05109665 9.84970692,8.85361317 C10.352422,8.45864665 11.05,7.71761367 11.05,6.375 C11.05,4.97425737 9.90074262,3.825 8.5,3.825 Z M8.5,11.9 C8.03055796,11.9 7.65,12.280558 7.65,12.75 C7.65,13.219442 8.03055796,13.6 8.5,13.6 C8.96944204,13.6 9.35,13.219442 9.35,12.75 C9.35,12.280558 8.96944204,11.9 8.5,11.9 Z" id="Shape"></path>
                                    </g>
                                </g>
                            </g>
                        </g>
                    </svg>
                </button>
                <div tabindex="-1">
                    <div class="components-popover components-dropdown__content bwf-ellipsis-menu__popover components-animate__appear is-from-top is-from-right is-without-arrow" data-x-axis="left" data-y-axis="bottom">
                        <div class="components-popover__content" tabindex="-1">
                            <div style="position: relative;">
                                <div role="menu" aria-orientation="vertical" class="bwf-ellipsis-menu__content">
									<?php
									foreach ( $navigation as $item ) {
										?>
                                        <a href="<?php echo esc_url($item['link']); ?>" target="<?php echo esc_attr($item['target']) ?>" role="menuitem" tabindex="0" class="bwf-ellipsis-menu__item">
                                            <div class="components-flex css-1ahbsz-Flex eboqfv50">
                                                <div class="components-flex__item css-1s295sp-Item eboqfv51">
													<?php
													$icon = ( isset( $item['icon'] ) && ! empty( $item['icon'] ) ) ? file_get_contents(  plugin_dir_path( WFFN_PLUGIN_FILE ) . 'admin/assets/img/menu/' . $item['icon'] . '.svg' )  : '';
													echo $icon;//phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
													?>
                                                </div>
                                                <div class="components-flex__block css-yr442k-Item-Block eboqfv52">
                                                    <div class="bwf_display_block">
                                                        <div class="menu-item-title"><?php echo esc_html($item['name']); ?></div>
                                                        <div class="menu-item-desc"><?php echo esc_html($item['desc']) ?></div>
                                                    </div>
                                                </div>
                                            </div>
                                        </a>
										<?php
									}
									?>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
		<?php


	}
	public function set_level_2_title( $title = '' ) {
		if ( empty( $title ) ) {
			return;
		}
		$this->data['level_2_title'] = $title;
	}

	public function set_level_2_post_title( $html = '' ) {
		if ( empty( $html ) ) {
			return;
		}
		$this->data['level_2_post_title'] = $html;
	}

	public function set_level_2_side_type( $type = 'menu' ) {
		if ( empty( $type ) ) {
			return;
		}
		$this->data['level_2_right_wrap_type'] = $type;
	}

	public function set_level_2_side_navigation( $navigation = [] ) {
		if ( empty( $navigation ) ) {
			return;
		}
		$this->data['level_2_right_side_navigation'] = $navigation;
	}

	public function set_level_2_navigation_pos( $positions = 'left' ) {
		if ( empty( $positions ) ) {
			return;
		}
		$this->data['level_2_navigation_pos'] = $positions;
	}
	public function set_level_2_side_navigation_active( $active = '' ) {
		if ( empty( $active ) ) {
			return;
		}
		$this->data['level_2_right_side_navigation_active'] = $active;
	}

	public function set_level_2_right_html( $html = '' ) {
		if ( empty( $html ) ) {
			return;
		}
		$this->data['level_2_right_html'] = $html;
	}



	public static function level_2_navigation_single_funnel( $funnel_id ) {
		return [
			'steps' => [
				'name' => __( 'Steps', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=bwf&path=/funnels/$funnel_id/funnel" ),
			],
			'contacts'     => [
				'name' => __( 'Contacts', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=bwf&path=/funnels/$funnel_id/contacts" ),
			],
			'analytics' => [
				'name' => __( 'Analytics', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=bwf&path=/funnels/$funnel_id/analytics" ),
				'isProFeature' => true,
				'showOnClick'  => true,
			],
			'settings'     => [
				'name' => __( 'Settings', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=bwf&path=/funnels/$funnel_id/settings" ),
			]
		];
	}

	public static function level_2_navigation_licenses() {
		return [
			'licenses' => [
				'name' => __( 'Licenses', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=woofunnels&tab=licenses" ),
			],
			'support'     => [
				'name' => __( 'Support', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=woofunnels&tab=support" ),
			],
			'tools' => [
				'name' => __( 'Tools', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=woofunnels&tab=tools" ),
			],
			'logs'     => [
				'name' => __( 'Logs', 'funnel-builder' ),
				'link' => admin_url( "admin.php?page=woofunnels&tab=logs" ),
			]
		];
	}


}
