<?php

/**
 * Class WFFN_Ecom_Tracking_Common
 */
if ( ! class_exists( 'WFFN_Ecomm_Tracking_Common' ) ) {
	class WFFN_Ecomm_Tracking_Common {
		public $api_events = [];

		public function __construct() {
			add_action( 'wp_head', array( $this, 'render' ), 90 );
			$this->admin_general_settings = BWF_Admin_General_Settings::get_instance();
		}

		public function should_render() {
			return apply_filters( 'wffn_allow_tracking', true, $this );
		}

		public function render() {

			if ( $this->should_render() ) {

				$this->render_fb();
				$this->render_ga();
				$this->maybe_render_conv_api();
			}
		}

		public function get_advanced_pixel_data() {
			return array();
		}

		public function render_fb() {
			if ( false !== $this->is_fb_pixel() ) {
				$fb_advanced_pixel_data = $this->get_advanced_pixel_data(); ?>
				<!-- Facebook Analytics Script Added By WooFunnels -->
				<script>
                    !function (f, b, e, v, n, t, s) {
                        if (f.fbq) return;
                        n = f.fbq = function () {
                            n.callMethod ?
                                n.callMethod.apply(n, arguments) : n.queue.push(arguments)
                        };
                        if (!f._fbq) f._fbq = n;
                        n.push = n;
                        n.loaded = !0;
                        n.version = '2.0';
                        n.queue = [];
                        t = b.createElement(e);
                        t.async = !0;
                        t.src = v;
                        s = b.getElementsByTagName(e)[0];
                        s.parentNode.insertBefore(t, s)
                    }(window, document, 'script', 'https://connect.facebook.net/en_US/fbevents.js');
					<?php

					$get_all_fb_pixel = $this->is_fb_pixel();
					$get_each_pixel_id = explode( ',', $get_all_fb_pixel );
					if ( is_array( $get_each_pixel_id ) && count( $get_each_pixel_id ) > 0 ) {
					foreach ( $get_each_pixel_id as $pixel_id ) {
					?>
					<?php if ( true === $this->is_fb_advanced_tracking_on() && count( $fb_advanced_pixel_data ) > 0 ) { ?>
                    fbq('init', '<?php echo esc_js( trim( $pixel_id ) ); ?>', <?php echo wp_json_encode( $fb_advanced_pixel_data ); ?>);
					<?php } else { ?>
                    fbq('init', '<?php echo esc_js( trim( $pixel_id ) ); ?>');
					<?php } ?>
					<?php
					}
					?>
					<?php  $this->render_fb_view(); ?>
					<?php  $this->maybe_print_fb_script(); ?>
					<?php
					}
					?>
				</script>
				<?php
			}
		}

		public function maybe_print_fb_script() {
			echo '';
		}

		public function is_fb_advanced_tracking_on() {
			$is_fb_advanced_tracking_on = $this->admin_general_settings->get_option( 'is_fb_advanced_event' );
			if ( is_array( $is_fb_advanced_tracking_on ) && count( $is_fb_advanced_tracking_on ) > 0 && 'yes' === $is_fb_advanced_tracking_on[0] ) {
				return true;
			}
		}

		public function should_render_view( $type ) { //phpcs:ignore VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedParameter,VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedVariable
			return false;
		}

		/**
		 * maybe render script to fire fb pixel view event
		 */
		public function render_fb_view() {
			if ( $this->should_render_view( 'fb' ) ) {
				?>
				var wffnEvents = [];
				var wfocu_ev_view_fb_event_id = Math.random().toString(36).substring(2, 15);
				fbq('track', 'PageView',{},{'eventID': 'PageView_'+wfocu_ev_view_fb_event_id});
				wffnEvents.push({event: 'PageView', 'event_id': 'PageView_'+wfocu_ev_view_fb_event_id});
				<?php

			}
		}

		public function is_fb_pixel() {

			$steps = WFFN_Core()->data->get_current_step();
			$key   = $this->admin_general_settings->get_option( 'fb_pixel_key' );

			if ( is_array( $steps ) && isset( $steps['id'] ) && get_post( $steps['id'] ) instanceof WP_Post ) {
				$setting = WFFN_Common::maybe_override_tracking( $steps['id'] );
				if ( is_array( $setting ) ) {
					$key = ( isset( $setting['fb_pixel_key'] ) && ! empty( $setting['fb_pixel_key'] ) ) ? $setting['fb_pixel_key'] : $key;
				}
			}

			$get_pixel_key = apply_filters( 'bwf_fb_pixel_ids', $key );

			return empty( $get_pixel_key ) ? false : $get_pixel_key;
		}


		/**
		 * render google analytics core script to load framework
		 */
		public function render_ga() {
			$get_tracking_code = $this->ga_code();
			if ( false === $get_tracking_code ) {
				return;
			}

			$get_tracking_code = explode( ",", $get_tracking_code );

			if ( ( $this->should_render_view( 'ga' ) ) && is_array( $get_tracking_code ) && count( $get_tracking_code ) > 0 ) {
				?>
				<!-- Google Analytics Script Added By WooFunnels -->
				<script>
					(function (window, document, src) {
						var a = document.createElement('script'),
							m = document.getElementsByTagName('script')[0];
						a.async = 1;
						a.src = src;
						m.parentNode.insertBefore(a, m);
					})(window, document, '//www.googletagmanager.com/gtag/js?id=<?php esc_js( trim( $get_tracking_code[0] ) ); ?>');

					window.dataLayer = window.dataLayer || [];
					window.gtag = window.gtag || function gtag() {
						dataLayer.push(arguments);
					};

					gtag('js', new Date());

					<?php
					foreach ( $get_tracking_code as $gad_code ) {
						echo "gtag('config', '" . esc_js( trim( $gad_code ) ) . "');";
					}

					$this->maybe_print_ga_script();
					?>
				</script>

				<?php
			}
		}

		/**
		 * Maybe print google analytics javascript
		 * @see WFFN_Ecomm_Tracking_Common::render_ga();
		 */
		public function maybe_print_ga_script() {
			echo '';
		}

		public function ga_code() {
			global $post;
			$step_id = $post->ID;
			$key     = $this->admin_general_settings->get_option( 'ga_key' );

			if ( $step_id > 0 && get_post( $step_id ) instanceof WP_Post ) {
				$setting = WFFN_Common::maybe_override_tracking( $step_id );
				if ( is_array( $setting ) ) {
					$key = ( isset( $setting['ga_key'] ) && ! empty( $setting['ga_key'] ) ) ? $setting['ga_key'] : $key;
				}
			}

			$get_ga_key = apply_filters( 'bwf_ga_key', $key );

			return empty( $get_ga_key ) ? false : $get_ga_key;
		}

		public function get_event_id( $event ) {
			return $event . "_" . time();
		}

		public function getRequestUri( $is_ajax = false ) {
			$request_uri = null;
			if ( true === $is_ajax ) {
				$current_step = WFFN_Core()->data->get_current_step();
				if ( is_array( $current_step ) && count( $current_step ) > 0 ) {
					return get_permalink( $current_step['id'] );
				}
			}
			if ( ! empty( $_SERVER['REQUEST_URI'] ) ) {
				$start       = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? "https" : "http" ) . "://";
				$request_uri = $start . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']; //phpcs:ignore
			}

			return $request_uri;
		}

		/**
		 * Is conversion API enabled from global settings
		 * @return bool
		 */
		public function is_conversion_api() {
			$is_conversion_api = $this->admin_general_settings->get_option( 'is_fb_purchase_conversion_api' );
			if ( is_array( $is_conversion_api ) && count( $is_conversion_api ) > 0 && 'yes' === $is_conversion_api[0] ) {
				return true;
			}

			return false;
		}

		/**
		 * Render a JS to fire async ajax calls to fire further events
		 */
		public function maybe_render_conv_api( $is_ajax = false ) {
			/**
			 * Special handling for the order received page
			 */
			if ( $this->is_conversion_api() ) {
				foreach ( $this->api_events as $event ) {
					$this->fire_conv_api_event( $event['event'], $event['event_id'], $is_ajax );
				}
			}
		}

		public function get_conversion_api_access_token() {

			$steps = WFFN_Core()->data->get_current_step();
			$key   = $this->admin_general_settings->get_option( 'conversion_api_access_token' );

			if ( is_array( $steps ) && isset( $steps['id'] ) && get_post( $steps['id'] ) instanceof WP_Post ) {
				$setting = WFFN_Common::maybe_override_tracking( $steps['id'] );
				if ( is_array( $setting ) ) {
					$key = ( isset( $setting['conversion_api_access_token'] ) && ! empty( $setting['conversion_api_access_token'] ) ) ? $setting['conversion_api_access_token'] : $key;
				}
			}

			$get_pixel_key = apply_filters( 'bwf_conversion_api_access_token', $key );

			return empty( $get_pixel_key ) ? false : $get_pixel_key;
		}

		public function get_conversion_api_test_event_code() {

			$steps = WFFN_Core()->data->get_current_step();
			$key   = $this->admin_general_settings->get_option( 'conversion_api_test_event_code' );

			if ( is_array( $steps ) && isset( $steps['id'] ) && get_post( $steps['id'] ) instanceof WP_Post ) {
				$setting = WFFN_Common::maybe_override_tracking( $steps['id'] );
				if ( is_array( $setting ) ) {
					$key = ( isset( $setting['conversion_api_test_event_code'] ) && ! empty( $setting['conversion_api_test_event_code'] ) ) ? $setting['conversion_api_test_event_code'] : $key;
				}
			}

			$get_pixel_key = apply_filters( 'bwf_conversion_api_test_event_code', $key );

			return empty( $get_pixel_key ) ? false : $get_pixel_key;
		}

		/**
		 * Ajax callback modal method to handle firing of multiple events in conv api
		 *
		 * @param string $type event name
		 *
		 * @return null
		 */
		public function fire_conv_api_event( $type, $event_id, $is_ajax = false ) {
			$get_all_fb_pixel  = $this->is_fb_pixel();
			$get_each_pixel_id = explode( ',', $get_all_fb_pixel );

			if ( is_array( $get_each_pixel_id ) && count( $get_each_pixel_id ) > 0 ) {

				foreach ( $get_each_pixel_id as $key => $pixel_id ) {


					$access_token = $this->get_conversion_api_access_token();
					$access_token = explode( ',', $access_token );

					if ( is_array( $access_token ) && count( $access_token ) > 0 ) {
						if ( isset( $access_token[ $key ] ) ) {

							BWF_Facebook_Sdk_Factory::setup( trim( $pixel_id ), trim( $access_token[ $key ] ) );
						}
					}

					$get_test      = $this->get_conversion_api_test_event_code();
					$get_test      = explode( ',', $get_test );
					$is_test_event = $this->admin_general_settings->get_option( 'is_fb_conv_enable_test' );
					if ( is_array( $is_test_event ) && count( $is_test_event ) > 0 && $is_test_event[0] === 'yes' && is_array( $get_test ) && count( $get_test ) > 0 ) {
						if ( isset( $get_test[ $key ] ) && ! empty( $get_test[ $key ] ) ) {
							BWF_Facebook_Sdk_Factory::set_test( trim( $get_test[ $key ] ) );
						}
					}

					BWF_Facebook_Sdk_Factory::set_partner( 'woofunnels' );
					$instance = BWF_Facebook_Sdk_Factory::create();
					if ( is_null( $instance ) ) {
						return null;
					}

					switch ( $type ) {
						case 'PageView':
							$instance->set_event_id( $event_id );
							$instance->set_user_data( $this->get_user_data( $type ) );
							$instance->set_event_source_url( $this->getRequestUri( $is_ajax ) );
							$instance->set_event_data( 'PageView', [] );
							break;
						case 'Purchase':
							$instance->set_event_source_url( $this->getRequestUri( $is_ajax ) );
							$instance->set_event_id( $event_id );
							$instance->set_user_data( $this->get_user_data( $type ) );
							$instance->set_event_data( 'Purchase', $this->get_purchase_params() );
							break;
						case 'trackCustom':
							$instance->set_event_id( $event_id );
							$instance->set_user_data( $this->get_user_data( $type ) );
							$instance->set_event_source_url( $this->getRequestUri( $is_ajax ) );
							$getEventName   = $this->admin_general_settings->get_option( 'general_event_name' );
							$getEventparams = $this->get_generic_event_params_for_conv_api();
							$instance->set_event_data( $getEventName, $getEventparams );
							break;
					}


					$response = $instance->execute();
					if ( $type === 'Purchase' ) {
						$this->maybe_insert_log( '----Facebook conversion API-----------' . print_r( $response, true ) ); //phpcs:ignore WordPress.PHP.DevelopmentFunctions.error_log_print_r

					}


				}
			}
		}
		/************************************ Conversion API related methods starts here ***************************/
		/**
		 * Maybe insert logs for the conversion API
		 *
		 * @param string $content
		 */
		public function maybe_insert_log( $content ) {

			if ( $this->is_enabled_log() ) {
				wc_get_logger()->log( 'info', $content, array( 'source' => 'bwf_facebook_conversion_api' ) );
			}
		}

		/**
		 * Check if logs are enabled or not for the conversion API
		 * @return bool
		 */
		public function is_enabled_log() {
			$is_conversion_api_log = $this->admin_general_settings->get_option( 'is_fb_conversion_api_log' );
			if ( is_array( $is_conversion_api_log ) && count( $is_conversion_api_log ) > 0 && 'yes' === $is_conversion_api_log[0] ) {
				return true;
			}

			return false;
		}

		/**
		 * Get current hour in the format supported by Facebook
		 * @return string string
		 */
		public function getHour() {
			$array = [
				'00-01',
				'01-02',
				'02-03',
				'03-04',
				'04-05',
				'05-06',
				'06-07',
				'07-08',
				'08-09',
				'09-10',
				'10-11',
				'11-12',
				'12-13',
				'13-14',
				'14-15',
				'15-16',
				'16-17',
				'17-18',
				'18-19',
				'19-20',
				'20-21',
				'21-22',
				'22-23',
				'23-24'
			];

			return $array[ gmdate( "G" ) ];

		}

		public function get_user_data( $type ) { //phpcs:ignore VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedParameter,VariableAnalysis.CodeAnalysis.VariableAnalysis.UnusedVariable
			$user_data                      = [];
			$user_data['client_ip_address'] = wffn_get_ip_address();
			$user_data['client_user_agent'] = wffn_get_user_agent();
			if ( isset( $_COOKIE['_fbp'] ) && ! empty( $_COOKIE['_fbp'] ) ) {
				$user_data['_fbp'] = wffn_clean( $_COOKIE['_fbp'] ); //phpcs:ignore WordPressVIPMinimum.Variables.RestrictedVariables.cache_constraints___COOKIE
			}
			if ( isset( $_COOKIE['_fbc'] ) && ! empty( $_COOKIE['_fbc'] ) ) {
				$user_data['_fbc'] = wffn_clean( $_COOKIE['_fbc'] ); //phpcs:ignore WordPressVIPMinimum.Variables.RestrictedVariables.cache_constraints___COOKIE
			}

			return $user_data;
		}

		public function maybe_ecomm_events( $events ) {

			$this->api_events = $events;
			$this->maybe_render_conv_api( true );
		}


	}
}

